local KM_GroundMounts = {};
local KM_WaterGroundMounts = {};
local KM_UnderWaterMounts = {};
local KM_NoSkillMounts = {};
local KM_FlyingMounts = {};
local KM_AQMounts = {};
local KM_LastCheck = GetTime();
local KMDebug = false;
local MMFavGround, MMFavFlying = {}, {};
local hasSeahorse, hasPoseidus = false, false
local PlayerFaction = select(1, UnitFactionGroup("player"));
local KMname = "\124c0c5f94ffKMount: \124cffffffff";
local KMnamedb = "\124c9021cfffKMount debug: \124cffffffff";
PlayerFaction = PlayerFaction == "Horde" and 0 or PlayerFaction == "Alliance" and 1;

function KM_GetMounts()
    KM_GroundMounts = {};
    KM_FlyingMounts = {};
    KM_UnderWaterMounts = {};
    KM_NoSkillMounts = {};
    KM_WaterGroundMounts = {};
    MMFavGround = {};
    MMFavFlying = {};

    KM_LastCheck = GetTime();

    for i = 1, C_MountJournal.GetNumMounts(), 1 do
        local creatureName, creatureID, icon, active, summonable, source, isFavorite, isFactionSpecific, faction, unknown, owned = C_MountJournal.GetMountInfo(i);
        local _,_,_,_, mountType = C_MountJournal.GetMountInfoExtra(i)

        --typeID = bit.band(3, typeID)
        if owned and (faction == nil or faction == PlayerFaction) and summonable == true then
            if mountType == 248 then
                if isFavorite == true then
                    table.insert(MMFavFlying, i);
                end
                table.insert(KM_FlyingMounts, i);
            end

            if mountType == 230 or mountType == 269 then
                table.insert(KM_GroundMounts, i);

                if isFavorite == true then
                    table.insert(MMFavGround, i);
                end

                if mountType == 269 then
                    table.insert(KM_WaterGroundMounts, i);
                end
            end

            if (mountType == 232 or mountType == 231 or mountType == 254) and creatureID ~= 75207 and creatureID ~= 98718 then
                table.insert(KM_UnderWaterMounts, i);
            end

            if mountType == 241 then
                if creatureID == 26656 then
                    table.insert(KM_GroundMounts, i);

                    if isFavorite == true then
                        table.insert(MMFavGround, i);
                    end
                end

                table.insert(KM_AQMounts, i);
            end

            -- Since the seahorse and reins of Poseidus is faster than sea turtle we can't just pick a underwater mount at random
            -- so we have to manually check for them.

            if creatureID == 75207 then
                hasSeahorse = i
            end

            if creatureID == 98718 then
                hasPoseidus = i
            end

            if mountType == 284 or creatureID == 30174 then
                if isFavorite == true then
                    table.insert(MMFavGround, i);
                end
                table.insert(KM_NoSkillMounts, i);
            end
        end
    end
end

-- Types are 1 for ground mount and 2 for flying mount
function KM_Mount(forceType)
    local CanFly, IsUnderwater = false, false;
    
    if KMDebug == true then
        print(KMnamedb .. "(" .. GetTime() .. ") Attempting to mount." .. (forceType ~= nil and " ForceType is set to " .. forceType .. "." or ""));
    end
    
    if IsSwimming() == true and select(1, GetMirrorTimerInfo(2)) == "BREATH" then
        -- I.e. if player is swimming and breath timer is active we assume they are underwater. Of course using underwater potions and so on messes this up.
        IsUnderwater = true;
    end

    if IsMounted() and IsFlying() and kmountdb["fdismount"] ~= true then
        return;
    end
    
    if IsMounted() == true and forceType == nil then
        Dismount();
        return;
    end

    if GetTime() > KM_LastCheck + 30 then
        KM_GetMounts();
    end

    if IsIndoors() == true then
        -- IsIndoors() returns nil if it is possible to mount, even if the location is indoors.
        UIErrorsFrame:AddMessage("KMount: You are indoors.", 1.0, 0.0, 0.0, 53, 5);
        return;
    end

    if HasFullControl() ~= true then
        -- Fear, cut scenes and so on
        UIErrorsFrame:AddMessage("KMount: Player not in full controll.", 1.0, 0.0, 0.0, 53, 5);
        return;
    end

    if UnitAffectingCombat("player") == true then
        UIErrorsFrame:AddMessage("KMount: You are in combat.", 1.0, 0.0, 0.0, 53, 5);
        return;
    end

    if GetSpellBookItemInfo("Apprentice Riding") == nil and GetSpellBookItemInfo("Journeyman Riding") == nil and GetSpellBookItemInfo("Expert Riding") == nil and GetSpellBookItemInfo("Artisan Riding") == nil and GetSpellBookItemInfo("Master Riding") == nil then
        if #KM_NoSkillMounts > 0 then
            -- Mounts requiring no riding skill. I.e. Riding Turtle and Chauffeured Mekgineer's Chopper/Mechano-Hog
            C_MountJournal.Summon(KM_NoSkillMounts[math.floor(math.random()*#KM_NoSkillMounts)]);
        else
            UIErrorsFrame:AddMessage("KMount: You don't have any riding skill.", 1.0, 0.0, 0.0, 53, 5);
            return;
        end
    end

    -- GetSpellBookItemInfo

    -- Map has to be set to current location or be closed for GetCurrentMapContinent() to actually return the player's location
    -- Otherwise it will return the continent index of whatever continent the player is looking at on the map
    SetMapToCurrentZone();

    if (GetCurrentMapAreaID() == 610 or GetCurrentMapAreaID() == 613 or GetCurrentMapAreaID() == 614 or GetCurrentMapAreaID() == 615) and IsUnderwater == true and (hasPoseidus ~= false or hasSeahorse ~= false) then
        -- While in waters of Vashj'ir
        if hasPoseidus ~= false then
            C_MountJournal.Summon(hasPoseidus);
        elseif hasSeahorse ~= false then
            C_MountJournal.Summon(hasSeahorse);
        end

        return;
    end

    if IsUnderwater == true and #KM_UnderWaterMounts > 0 and kmountdb["wmount"] == true then
        -- While in water
        C_MountJournal.Summon(KM_UnderWaterMounts[math.floor(math.random()*#KM_UnderWaterMounts)  + 1]);
        return;
    end

    if GetCurrentMapAreaID() == 772 and #KM_AQMounts > 0 then
        -- Ahn'Qiraj
        C_MountJournal.Summon(KM_AQMounts[math.floor(math.random()*#KM_AQMounts) + 1]);
        return;
    end

    if GetCurrentMapContinent() == 1 or GetCurrentMapContinent() == 2 or GetCurrentMapAreaID() == 640 and GetSpellBookItemInfo("Flight Master's License") ~= nil then
        -- Kalimdor, Eastern Kingdoms and Deepholm
        CanFly = true;
    elseif GetCurrentMapContinent() == 3 and (GetSpellBookItemInfo("Expert Riding") ~= nil or GetSpellBookItemInfo("Artisan Riding") ~= nil or GetSpellBookItemInfo("Master Riding") ~= nil) then
        -- Outland
        CanFly = true;
    elseif GetCurrentMapContinent() == 4 and GetSpellBookItemInfo("Cold Weather Flying") ~= nil then
        -- Northrend
        CanFly = true;
    elseif GetCurrentMapContinent() == 6 and GetSpellBookItemInfo("Wisdom of The Four Winds") ~= nil then
        -- Pandaria
        CanFly = true;
    elseif GetCurrentMapContinent() == 7 and select(4, GetAchievementInfo(10018)) == true then
        -- Draenor
        CanFly = true;
    end

    if ((CanFly == true and IsFlyableArea() == true and IsUnderwater ~= true) or forceType == 2) and forceType ~= 1 then
        -- Flying mount
        if forceType == 2 and #KM_FlyingMounts == 0 then
            print(KMname .. "Tried forcing flying mount, but no such mount available.");
            return;
        end
        
        if kmountdb["UseBlizFav"] == true and #MMFavFlying > 0 then
            C_MountJournal.Summon(MMFavFlying[math.floor(math.random()*#MMFavFlying) + 1]);
        elseif kmountdb["PreferredFMount"] ~= nil then
            C_MountJournal.Summon(kmountdb["PreferredFMount"]);
        else
            C_MountJournal.Summon(KM_FlyingMounts[math.floor(math.random()*#KM_FlyingMounts) + 1]);
        end
    else
        -- Regular mount
        if #KM_WaterGroundMounts > 0 and kmountdb["preferwmount"] == true then
            -- Unless player is Death Knight we prioritize mounts that can walk on water as they're more convenient
            C_MountJournal.Summon(KM_WaterGroundMounts[math.floor(math.random()*#KM_WaterGroundMounts) + 1]);
        elseif kmountdb["UseBlizFav"] == true and #MMFavGround > 0 then
            C_MountJournal.Summon(MMFavGround[math.floor(math.random()*#MMFavGround) + 1]);
        elseif kmountdb["PreferredGMount"] ~= nil then
            C_MountJournal.Summon(kmountdb["PreferredGMount"]);
        else
            C_MountJournal.Summon(KM_GroundMounts[math.floor(math.random()*#KM_GroundMounts) + 1]);
        end
    end
end

local function indexOf(t, object)
    if type(t) == "table" then
        for i = 1, #t do
            if object == t[i] then
                return i
            end
        end
        return -1
    end
end

local function isValidMount(n, t)
    -- t is type of mount. 1 = Ground, 2 = Flying

    KM_GetMounts();

    n = tonumber(n);

    if C_MountJournal.GetMountInfo(n) == nil then
        print(KMname .. "Invalid mount index.");
        return false;
    end

    if select(9, C_MountJournal.GetMountInfo(n)) ~= nil and select(9, C_MountJournal.GetMountInfo(n)) ~= PlayerFaction then
        -- Wrong faction
        print(KMname .. "'" .. select(1, C_MountJournal.GetMountInfo(n)) .. "' is not usable by " .. select(1, UnitFactionGroup("player")) .. " characters.");
        return false;
    end

    if t == 1 and indexOf(KM_GroundMounts, n) == -1 and indexOf(KM_NoSkillMounts, n) == -1 then
        -- Check if mount is in array of valid ground mounts
        print(KMname .. "'" .. select(1, C_MountJournal.GetMountInfo(n)) .. "' is not available as a ground mount for this character.");
        return false;
    end

    if t == 2 and indexOf(KM_FlyingMounts, n) == -1 then
        -- Check if mount is in array of valid flying mounts
        print(KMname .. "'" .. select(1, C_MountJournal.GetMountInfo(n)) .. "' is not available as a flying mount for this character.");
        return false;
    end

    return true;
end

local function handler(msg, editbox)
    local command, rest = msg:match("^(%S*)%s*(.-)$");

    if command == "get" then
        if rest == nil then
            print("Missing type. Valid types are 'ground' and 'flying'.");
            return;
        end

        local mtype = rest == "ground" and 1 or rest == "flying" and 2 or false;

        if mtype == false then
            print("Valid types are 'ground' and 'flying'.");
            return;
        end

        local count = 0;
        
        for i = 1, C_MountJournal.GetNumMounts(), 1 do
            local creatureName, _,_,_, summonable,_, isFavorite,_, faction,_, owned = C_MountJournal.GetMountInfo(i);
            local _,_,_,_, mountType = C_MountJournal.GetMountInfoExtra(i)

            --typeID = bit.band(3, typeID)
            if owned and summonable and(faction == nil or faction == PlayerFaction) then
                if mountType == 248 and mtype == 2 then
                    count = count + 1;
                    print("\124c0c5f94ff" .. creatureName .. "\124cffffffff: " .. i .. (isFavorite == true and " *" or isFavorite == false and ""));
                elseif (mountType == 230 or mountType == 269) and mtype == 1 then
                    count = count + 1;
                    print("\124c0c5f94ff" .. creatureName .. "\124cffffffff: " .. i .. (isFavorite == true and " *" or isFavorite == false and ""));
                end
            end
        end
        
        if count == 0 then
            print(KMname .. "No mounts available in that category.");
        end
    elseif command == "getbyname" then
        for i = 1, C_MountJournal.GetNumMounts(), 1 do
            local creatureName, _,_,_,_,_, _,_, _,_, _ = C_MountJournal.GetMountInfo(i);
            if strlower(strsub(creatureName, 1, strlen(rest))) == strlower(rest) then
                print(KMname .. "Index of '" .. creatureName .. "' is '" .. i .. "'.");
                return;
            end
        end
        print(KMname .. " '" .. rest .. "' did not match any mounts.");
    elseif command == "setg" then
        if isValidMount(rest, 1) then
            kmountdb["PreferredGMount"] = rest;
            print(KMname .. "Ground mount preference set to " .. select(1, C_MountJournal.GetMountInfo(kmountdb["PreferredGMount"])) .. ".");
        end
    elseif command == "setf" then
        if isValidMount(rest, 2) then
            kmountdb["PreferredFMount"] = rest;
            print(KMname .. "Flying mount preference set to " .. select(1, C_MountJournal.GetMountInfo(kmountdb["PreferredFMount"])) .. ".");
        end
    elseif command == "usetf" then
        kmountdb["PreferredFMount"] = nil;
        print(KMname .. "Unset preferred flying mount.");
    elseif command == "usetg" then
        kmountdb["PreferredGMount"] = nil;
        print(KMname .. "Unset preferred ground mount.");
    elseif command == "togglefdismount" then
        if kmountdb["fdismount"] == true then
            kmountdb["fdismount"] = false;
            print(KMname .. "Dismount while flying set to false.");
        else
            kmountdb["fdismount"] = true;
            print(KMname .. "Dismount while flying set to true.");
        end
    elseif command == "togglew" then
        if kmountdb["wmount"] == true then
            kmountdb["wmount"] = false;
            print(KMname .. "Water mount preference set to false.");
        else
            kmountdb["wmount"] = true;
            print(KMname .. "Water mount preference set to true.");
        end
    elseif command == "togglepw" then
        if kmountdb["preferwmount"] == true then
            kmountdb["preferwmount"] = false;
            print(KMname .. "Water walking ground mount preference set to false.");
        else
            kmountdb["preferwmount"] = true;
            print(KMname .. "Water walking ground mount preference set to true.");
        end
    elseif command == "info" then
        print("\124c06418affKek\124c34ec7900Mount")
        print("\124c0c5f94ffPreferred Ground Mount: \124cffffffff" .. (kmountdb["PreferredGMount"] ~= nil and select(1, C_MountJournal.GetMountInfo(kmountdb["PreferredGMount"])) or "None"));
        print("\124c0c5f94ffPreferred Flying Mount: \124cffffffff" .. (kmountdb["PreferredFMount"] ~= nil and select(1, C_MountJournal.GetMountInfo(kmountdb["PreferredFMount"])) or "None"));
        print("\124c0c5f94ffDismount in flight: \124cffffffff" .. (kmountdb["fdismount"] == true and "True" or "False"));
        print("\124c0c5f94ffUse Water Mount(s): \124cffffffff" .. (kmountdb["wmount"] == true and "True" or "False"));
        print("\124c0c5f94ffUse mount manager favorite(s): \124cffffffff" .. (kmountdb["UseBlizFav"] == true and "True" or "False"));
        print("\124c0c5f94ffPrefer water walking mount(s): \124cffffffff" .. (kmountdb["preferwmount"] == true and "True" or "False"));
    elseif command == "reset" then
        kmountdb = {};
        kmountdb["wmount"] = true;
        kmountdb["fdismount"] = true;
        kmountdb["UseBlizFav"] = false;
        if select(3, UnitClass("player")) ~= 6 then
            kmountdb["preferwmount"] = true;
        end
        print("\124c0c5f94ffKMount:\124cffffffff Settings for '" .. UnitName("player") .. "' have been reset.");
    elseif command == "g" then
        KM_Mount(1);
    elseif command == "f" then
        KM_Mount(2);
    elseif command == "debug" then
        if KMDebug == true then
            KMDebug = false;
            print(KMname .. "Debug mode deactivated");
        else
            KMDebug = true;
            print(KMname .. "Debug mode activated");
        end
    elseif command == "toggleblizfav" then
        if kmountdb["UseBlizFav"] == true then
            kmountdb["UseBlizFav"] = false;
            print(KMname .. "Use mount manager favorite mount(s) set to false.");
        else
            kmountdb["UseBlizFav"] = true;
            print(KMname .. "Use mount manager favorite mount(s) set to true.");
        end
    elseif command == "help" then
        print("\124c06418affKek\124c34ec7900Mount")
        print("\124c0c5f94ffGet <type> - \124cffffffffPrints all owned, usable mounts. Usefull to find the index for specific mounts.");
        print("\124c0c5f94ffGetbyname <query> - \124cffffffffSearch through all mounts (even unavailable) and returns an index number if a match is found.");
        print("\124c0c5f94ffSetg - \124cffffffffSet the desired ground mount.");
        print("\124c0c5f94ffUsetg - \124cffffffffUnsets the desired ground mount.");
        print("\124c0c5f94ffSetf - \124cffffffffSet the desired flying mount.");
        print("\124c0c5f94ffUsetf - \124cffffffffUnsets the desired flying mount.");
        print("\124c0c5f94ffTogglew - \124cffffffffToggles water mounts (except Vashj'ir specific mounts).");
        print("\124c0c5f94ffTogglepw - \124cffffffffToggles preference for ground mounts that can walk on water.");
        print("\124c0c5f94ffTogglefdismount - \124cffffffffToggles dismount preference while flying.");
        print("\124c0c5f94ffToggleBlizFav - \124cffffffffToggles use of mount manager favorite(s).");
        print("\124c0c5f94ffF - \124cffffffffForce summon flying mount.");
        print("\124c0c5f94ffF - \124cffffffffForce summon ground mount.");
        print("\124c0c5f94ffInfo - \124cffffffffDisplays settings for current character.");
        print("\124c0c5f94ffReset - \124cffffffffReset settings for current character.");
        print("\124c0c5f94ffDebug - \124cffffffffToggle debug mode.");
        print("\124c0c5f94ffHelp - \124cffffffffShow this list.");
        print("\124cffffffffFor more info visit curse.");
    else
        KM_Mount();
    end
end

do
    if kmountdb == nil then
        kmountdb = {};
        kmountdb["wmount"] = true;
        kmountdb["fdismount"] = true;
        kmountdb["UseBlizFav"] = false;
        if select(3, UnitClass("player")) ~= 6 then
            kmountdb["preferwmount"] = true;
        end
    end
    KM_GetMounts();
    -- Slash Handler
    SLASH_KekMount1 = "/kmount"
    SlashCmdList.KekMount = handler
end